//
//	Encoder for G.729A CS-ACELP Speech Codec
// 
//		pre_processing.cpp : Module for 
//			- reading the 8kHz, 16-bit PCM .wav file
//			- saving intermediate files
//

#include "StdAfx.h"
#include "file_io.h"


int getLength(FILE *Infile)
{
	unsigned char *len;
	int l = 0;

	if (Infile != NULL)
		{
		len = (unsigned char *) malloc(4);
		if (!fseek(Infile, 54, SEEK_SET))
		{
			fread(len, sizeof(unsigned char), 4, Infile);
			l = ((len[0]&0xff)          + (len[1]&0xff)*16*16
				+(len[2]&0xff)*16*16*16 + (len[3]&0xff)*16*16*16*16);
		}
		else
			l = -1;
	}
	return l;
}


//Check if the header of the wave file is correct
int checkHeader(FILE *Infile)
{
	int correct = 1;
	char len[4];
	char riff[4];
	char wave[4];
	char fmt[4];
	char fact[4];
	char data[4];
	int fileLen, fmtLen, factLen, dataLen;
	int tag;
	int channel;
	int samplerate;
	int bpsec, bpsam;
	int blockalign;
	int factdata;

	//Reading out all the info
	fseek(Infile, 0, SEEK_SET);
	
	//reading "RIFF"
	fread(riff, sizeof(char), 4, Infile);
	if (riff[0]!='R' || riff[1]!='I' || riff[2]!='F' || riff[3]!='F')
		correct = 0;
	else
	{
		//reading and get the file size
		fread(len, sizeof(char), 4, Infile);
		fileLen = ((len[0]&0xff)          + (len[1]&0xff)*SIXTEEN2
				  +(len[2]&0xff)*SIXTEEN4 + (len[3]&0xff)*SIXTEEN6);

		//reading "WAVE"
		fread(wave, sizeof(char), 4, Infile);
		//reading "fmt "
		fread(fmt, sizeof(char), 4, Infile);
		
		if (wave[0]!='W' || wave[1]!='A' || wave[2]!='V' || wave[3]!='E' ||
			fmt[0]!='f' || fmt[1]!='m' || fmt[2]!='t' || fmt[3]!=' ')
			correct = 0;
		else
		{
			//reading and get fmtLen, tag, channel, samplerate, ....
			fread(len, sizeof(char), 4, Infile);
			fmtLen = ((len[0]&0xff)          + (len[1]&0xff)*SIXTEEN2
				     +(len[2]&0xff)*SIXTEEN4 + (len[3]&0xff)*SIXTEEN6);
			fread(len, sizeof(char), 2, Infile);
			tag     = ((len[0]&0xff) + (len[1]&0xff)*SIXTEEN2);
			fread(len, sizeof(char), 2, Infile);
			channel = ((len[0]&0xff) + (len[1]&0xff)*SIXTEEN2);
			fread(len, sizeof(char), 4, Infile);
			samplerate = ((len[0]&0xff)          + (len[1]&0xff)*SIXTEEN2
						 +(len[2]&0xff)*SIXTEEN4 + (len[3]&0xff)*SIXTEEN6);
			fread(len, sizeof(char), 4, Infile);
			bpsec = ((len[0]&0xff)          + (len[1]&0xff)*SIXTEEN2
					+(len[2]&0xff)*SIXTEEN4 + (len[3]&0xff)*SIXTEEN6);
			fread(len, sizeof(char), 2, Infile);
			blockalign = ((len[0]&0xff) + (len[1]&0xff)*SIXTEEN2);
			fread(len, sizeof(char), 2, Infile);
			bpsam      = ((len[0]&0xff) + (len[1]&0xff)*SIXTEEN2);

			//reading "fact"
			fread(fact, sizeof(char), 2, Infile);
			fread(fact, sizeof(char), 4, Infile);
			if (fact[0]!='f' || fact[1]!='a' || fact[2]!='c' || fact[3]!='t')
				correct = 0;
			else
			{
				fread(len, sizeof(char), 4, Infile);
				factLen = ((len[0]&0xff)          + (len[1]&0xff)*SIXTEEN2
					      +(len[2]&0xff)*SIXTEEN4 + (len[3]&0xff)*SIXTEEN6);
				fread(len, sizeof(char), 4, Infile);
				factdata = ((len[0]&0xff)          + (len[1]&0xff)*SIXTEEN2
						   +(len[2]&0xff)*SIXTEEN4 + (len[3]&0xff)*SIXTEEN6);
				fread(data, sizeof(char), 4, Infile);
				if (data[0]!='d' || data[1]!='a' || data[2]!='t' || data[3]!='a')
					correct = 0;
				else
				{
					fread(len, sizeof(char), 4, Infile);
					dataLen = ((len[0]&0xff)         + (len[1]&0xff)*SIXTEEN2
							 +(len[2]&0xff)*SIXTEEN4 + (len[3]&0xff)*SIXTEEN6);
				}
			}
		}
	}

	printf("sampling rate = %dkHz\n", samplerate/1000);
	printf("number of bits per sample = %d\n", bpsam);
	printf("number of samples = %d\n", factdata);

	if (samplerate != 8000 || channel != 1)
		correct = 0;
	return correct;
}


//Create the wave file header information
void createHeader(FILE *Outfile)
{
	fwrite("RIFF", sizeof(char), 4, Outfile);
	fwrite("\x00\x00\x00\x00", sizeof(char), 4, Outfile);   //file size
	fwrite("WAVEfmt ", sizeof(char), 8, Outfile);
	fwrite("\x12\x00\x00\x00", sizeof(char), 4, Outfile);	//fmt length
	fwrite("\x01\x00\x01\x00", sizeof(char), 4, Outfile);   //tage and channel
	fwrite("\x40\x1F\x00\x00", sizeof(char), 4, Outfile);	//sample rate
	fwrite("\x80\x3E\x00\x00", sizeof(char), 4, Outfile);	//bits/second
	fwrite("\x02\x00\x10\x00", sizeof(char), 4, Outfile);   //blockalign and bits/sample
	fwrite("\x00\x00", sizeof(char), 2, Outfile);
	fwrite("fact", sizeof(char), 4, Outfile);
	fwrite("\x04\x00\x00\x00", sizeof(char), 4, Outfile);	//fact length
	fwrite("\x00\x00\x00\x00", sizeof(char), 4, Outfile);	//sizeof data/2
	fwrite("data", sizeof(char), 4, Outfile);
	fwrite("\x00\x00\x00\x00", sizeof(char), 4, Outfile);	//sizeof data
}


void writeFilesize(FILE *Outfile, int fileLen)
{
	unsigned char bit0, bit1, bit2, bit3;

	int len = fileLen*2+50;
	char s[] = "0000";

	bit3 = len / SIXTEEN6;
	len  = len % SIXTEEN6;
	bit2 = len / SIXTEEN4;
	len  = len % SIXTEEN4;
	bit1 = len / SIXTEEN2;
	len  = len % SIXTEEN2;
	bit0 = len;
	fseek(Outfile, 4, SEEK_SET);
	fprintf(Outfile, "%c%c%c%c", bit0, bit1, bit2, bit3);

	len  = fileLen;
	bit3 = len / SIXTEEN6;
	len  = len % SIXTEEN6;
	bit2 = len / SIXTEEN4;
	len  = len % SIXTEEN4;
	bit1 = len / SIXTEEN2;
	len  = len % SIXTEEN2;
	bit0 = len;
	fseek(Outfile, 46, SEEK_SET);
	fprintf(Outfile, "%c%c%c%c", bit0, bit1, bit2, bit3);

	len  = fileLen*2;
	bit3 = len / SIXTEEN6;
	len  = len % SIXTEEN6;
	bit2 = len / SIXTEEN4;
	len  = len % SIXTEEN4;
	bit1 = len / SIXTEEN2;
	len  = len % SIXTEEN2;
	bit0 = len;
	fseek(Outfile, 54, SEEK_SET);
	fprintf(Outfile, "%c%c%c%c", bit0, bit1, bit2, bit3);

}
